package yrpg.bamp.base.plugin.task;

import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import kd.bos.context.RequestContext;
import kd.bos.dataentity.OperateOption;
import kd.bos.dataentity.RefObject;
import kd.bos.dataentity.entity.DynamicObject;
import kd.bos.entity.operate.result.OperationResult;
import kd.bos.exception.KDBizException;
import kd.bos.exception.KDException;
import kd.bos.orm.ORM;
import kd.bos.orm.query.QCP;
import kd.bos.orm.query.QFilter;
import kd.bos.schedule.executor.AbstractTask;
import kd.bos.servicehelper.BusinessDataServiceHelper;
import kd.bos.servicehelper.operation.OperationServiceHelper;
import kd.bos.servicehelper.operation.SaveServiceHelper;
import kd.bos.util.HttpClientUtils;
import kd.bos.util.StringUtils;
import kd.sdk.plugin.Plugin;

import java.io.IOException;
import java.util.ArrayList;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * 定时任务获取SAP内部订单
 */
public class PullSAPInnerOrderTask extends AbstractTask implements Plugin {

    @Override
    public void execute(RequestContext requestContext, Map<String, Object> map) throws KDException {
        //同步日期
        Object syncDate = map.get("syncDate");
        //同步公司
        Object syncOrg = map.get("syncOrg");
        //记录日期
        Map<String, Object> logMap = new HashMap<>();
        logMap.put("opname", "定时任务获取SAP内部订单");
        logMap.put("thirdappname", "SAP");
        logMap.put("status", "true");

        //预留入参
        String dateStr = syncDate==null?"":syncDate.toString();
        String orgStr = syncOrg==null?"":syncOrg.toString();
        //拼接参数
        JSONObject reqJson = new JSONObject();
        reqJson.put("functionName","Z_GET_ORDER_FY");
        reqJson.put("client","erp");
        reqJson.put("export","ZOUT");
        JSONObject inputParam = new JSONObject();
        inputParam.put("UDATE",dateStr);
        inputParam.put("I_BUKRS",orgStr);
        reqJson.put("inputParamMap",inputParam);
        reqJson.put("export","ZOUT");
        String url = "http://wapi2.yangzijiang.com:8089/sap/queryOrDoDataToRFC";

        logMap.put("opdesc",url);
        logMap.put("request", reqJson.toString());

        Map<String,String> header =new HashMap<>();
        header.put("Content-Type","application/json;charset=utf-8");
        //调用接口
        String reponseStr = "";
        try {
            reponseStr= HttpClientUtils.postjson(url,header,reqJson.toString(),20000,40000);
        } catch (IOException e) {
            throw new RuntimeException(e);
        }
        //解析返回值
        if(StringUtils.isEmpty(reponseStr)){
            logMap.put("response", "调用SAP内部订单中间服务接口失败，返回值为空！");
            logMap.put("status", "false");
            saveDetailApiLog(logMap,saveApiLog(logMap));
        }else{
            //获取现有数据
            QFilter qFilter = new QFilter("group.number", QCP.equals,"NBDD");
            DynamicObject[] assistantdatas = BusinessDataServiceHelper.load("bos_assistantdata_detail", "id,name,number,ctrlstrategy,yrpg_iotype,parent,group,createorg,ctrlview,isleaf,level", new QFilter[]{qFilter});
            Map<String,DynamicObject> infoMap = new HashMap<String,DynamicObject>();
            for(DynamicObject info:assistantdatas){
                infoMap.put(info.getString("number"),info);
            }
//            辅助资料分类-内部订单
            QFilter numFilter = new QFilter("number", QCP.equals,"NBDD");
            DynamicObject group = BusinessDataServiceHelper.loadSingle("bos_assistantdatagroup", new QFilter[]{numFilter});
            //解析辅助资料实体信息
            List<DynamicObject> infos = new ArrayList<DynamicObject>();
            List<DynamicObject> updateinfos = new ArrayList<DynamicObject>();
            JSONObject response = JSONObject.parseObject(reponseStr);
            JSONArray zout = response.getJSONArray("ZOUT");
            for(int i=0;i<zout.size();i++){
                JSONObject jsonObject = zout.getJSONObject(i);
                String orgNum = jsonObject.getString("BUKRS");
                String name = jsonObject.getString("KTEXT");
                if(StringUtils.isEmpty(name)){
                    continue;
                }
                String number = jsonObject.getString("AUFNR");
                //编码截取
                String type = jsonObject.getString("AUART");
                number = number.substring(6);
                //判断是否存在
                DynamicObject info = infoMap.get(number);
                if(info==null){
                    info = BusinessDataServiceHelper.newDynamicObject("bos_assistantdata_detail");
                    QFilter orgNumberFilter = new QFilter("yrpg_companynum", QCP.equals, orgNum);
                    DynamicObject orgInfo = BusinessDataServiceHelper.loadSingleFromCache("bos_adminorg", "id,name", new QFilter[]{orgNumberFilter});
                    if(orgInfo==null){
                        continue;
                    }
                    info.set("createorg",orgInfo);
                    info.set("group",group);
                    info.set("enable","1");
                    info.set("status","C");
                    info.set("number",number);
                    info.set("ctrlstrategy","6");
                    info.set("name",name);
                    info.set("yrpg_iotype",type);
                    infos.add(info);
                }else{
                    info.set("name",name);
                    info.set("yrpg_iotype",type);
                    updateinfos.add(info);
                }
            }
            if(updateinfos.size()>0){
                SaveServiceHelper.save(updateinfos.toArray(new DynamicObject[0]));
            }
            if(infos.size()>0){
                DynamicObject[] array = infos.toArray(new DynamicObject[0]);
                //执行保存并判断
                OperationResult bosAssistantdataDetail = OperationServiceHelper.executeOperate("save", "bos_assistantdata_detail", array, OperateOption.create());
                if(!bosAssistantdataDetail.isSuccess()){
                    throw new KDBizException(bosAssistantdataDetail.getAllErrorOrValidateInfo().toString());
                }
            }
        }

    }


    public static long saveApiLog(Map<String,Object> logMap){
        DynamicObject apiLog = BusinessDataServiceHelper.newDynamicObject("openapi_log_data");
        ORM.create().setPrimaryKey(apiLog);
        //api编号
        apiLog.set("opname",logMap.get("opname"));
        //URL
        apiLog.set("opdesc",logMap.get("opdesc"));
        //请求参数
        if(logMap.get("request")!=null){
            apiLog.set("request",logMap.get("request").toString().length()>2000?logMap.get("request").toString().substring(0, 2000):logMap.get("request"));
        }else {
            apiLog.set("request","");
        }

        if (logMap.get("response")!=null){
            //响应结果
            apiLog.set("response",logMap.get("response").toString().length()>2000?logMap.get("response").toString().substring(0, 2000):logMap.get("response"));
        }else {
            apiLog.set("response","");
        }
        //调用时间
        apiLog.set("opdate",new Date());
        //调用者
        apiLog.set("username",logMap.get("username"));
        //第三方应用
        apiLog.set("thirdappname",logMap.get("thirdappname"));
        //调用状态
        apiLog.set("status",logMap.get("status"));
        //保存日志
        SaveServiceHelper.save(new DynamicObject[]{apiLog});

        return apiLog.getLong("id");

    }


    public static void saveDetailApiLog(Map<String,Object> logMap,long id){
        DynamicObject apiLog = BusinessDataServiceHelper.newDynamicObject("openapi_log_detail");
        //调用者
        apiLog.set("username",logMap.get("username"));
        //调用时间
        apiLog.set("opdate",new Date());
        //入参
        apiLog.set("reqpara_tag",logMap.get("request").toString());
        //响应
        apiLog.set("resppara_tag",logMap.get("response").toString());

        apiLog.set("id",id);

        SaveServiceHelper.save(new DynamicObject[]{apiLog});

    }
}