package yrpg.bamp.base.webapi;

import javax.validation.Valid;
import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import kd.bos.dataentity.entity.DynamicObject;
import kd.bos.dataentity.entity.DynamicObjectCollection;
import kd.bos.openapi.api.result.ApiSaveResult;
import kd.bos.openapi.api.result.BaseFilterItemData;
import kd.bos.openapi.api.result.BaseFilterResult;
import kd.bos.openapi.common.custom.annotation.ApiController;
import kd.bos.openapi.common.custom.annotation.ApiParam;
import kd.bos.openapi.common.custom.annotation.ApiPostMapping;
import kd.bos.openapi.common.result.CustomApiResult;
import kd.bos.openapi.common.result.OpenApiResult;
import kd.bos.openapi.common.util.OpenApiSdkUtil;
import kd.bos.orm.query.QCP;
import kd.bos.orm.query.QFilter;
import kd.bos.servicehelper.QueryServiceHelper;
import kd.bos.util.StringUtils;
import yrpg.bamp.base.common.ApiReqModel;
import yrpg.bamp.base.common.AssembleResponseUtil;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

@ApiController(value="CommonBill",desc="统一url入口")
public class AdminApiController {
	//存放实际地址和接口入口的
	@ApiPostMapping(value="adminURLApiHandler")
	public CustomApiResult<Object> adminApiHandler(@Valid  @ApiParam("入口参数") ApiReqModel reqModel) {
		//获取入参
		String reqdata=reqModel.getData();
		JSONArray dataArray = JSONObject.parseArray(reqdata);

		// 新增（修改）、启用（解封）、禁用（封存） 入参
		JSONObject saveDataJson = new JSONObject();
		JSONObject freezeDataJson = new JSONObject();
		JSONObject unfreezeDataJson = new JSONObject();
		//解封时更新
		JSONObject unfreezeUpdateDataJson = new JSONObject();

		// 新增（修改）、启用（解封）、禁用（封存） data层参数
		JSONArray adminData = new JSONArray();
		JSONArray freezeNums = new JSONArray();
		JSONArray unfreezeNums = new JSONArray();
		//解封时更新
		JSONArray unfreezeData = new JSONArray();

		JSONObject freezeNumsJson = new JSONObject();
		JSONObject unfreezeNumsJson = new JSONObject();
		//取出所有请求部门
		List<String> mainids = new ArrayList<String>();
		for(int i=0;i<dataArray.size();i++){
			JSONObject jsonObject = dataArray.getJSONObject(i);
			String number = jsonObject.getString("yrpg_mainid");
			mainids.add(number);
		}
		//获取系统现有部门数据
		QFilter numFilter = new QFilter("structure.view.number", QCP.equals,"01");
		QFilter mainidFilter = new QFilter("yrpg_mainid", QCP.in,mainids);
		DynamicObjectCollection bosAdminorg = QueryServiceHelper.query("bos_adminorg", "number,yrpg_mainid,structure.isfreeze", new QFilter[]{numFilter,mainidFilter});
		Map<String,Boolean> adminStatusMap = new HashMap<String,Boolean>();
		Map<String,String> adminNumMap = new HashMap<String,String>();
		for (DynamicObject admin : bosAdminorg){
			adminStatusMap.put(admin.getString("yrpg_mainid"),admin.getBoolean("structure.isfreeze"));
			adminNumMap.put(admin.getString("number"),admin.getString("yrpg_mainid"));
		}
		//新增废弃数据，执行跳过，接口返回参数
		JSONArray newfreezeData = new JSONArray();

		//循环判断部门进入保存、封存、解封集合
		for(int i=0;i<dataArray.size();i++){
			JSONObject jsonObject = dataArray.getJSONObject(i);
			String number = jsonObject.getString("yrpg_mainid");
			boolean isfreeze = jsonObject.getBoolean("isfreeze");
			//根据入参是否封存，已经系统中状态判断
			if(adminStatusMap.get(number)==null&&isfreeze){
				newfreezeData.add(jsonObject);
				//封存新增直接跳过
				continue;
			}
			if(adminStatusMap.get(number)==null){
				//新增
				adminData.add(jsonObject);
			}else if(!isfreeze&&!adminStatusMap.get(number)){
				//更新
				adminData.add(jsonObject);
			}else if(!adminStatusMap.get(number)&&isfreeze){
				//封存
				freezeNums.add(number);
			}else if(adminStatusMap.get(number)&&!isfreeze){
				//反封存以及更新
				unfreezeNums.add(number);
				unfreezeData.add(jsonObject);
			}
		}

		//返回值
		BaseFilterResult responseData = new BaseFilterResult();
		CustomApiResult finalResult =new CustomApiResult();
		//判断是否调用封存解封
		boolean isfreeze = false;
		//调用封存接口
		if(freezeNums.size()>0){
			isfreeze = true;
			freezeNumsJson.put("yrpg_mainids",freezeNums);
			freezeDataJson.put("data",freezeNumsJson);
			//调用接口
			OpenApiResult adminfreeze = OpenApiSdkUtil.invoke("/v2/yrpg/base/bos_adminorg/adminfreeze", freezeDataJson, new HashMap<>());
			BaseFilterResult data = (BaseFilterResult)adminfreeze.getData();
			List<BaseFilterItemData> result = data.getResult();
			for(BaseFilterItemData item : result){
				String number = item.getNumber();
				if(!StringUtils.isEmpty(number)){
					item.setNumber(adminNumMap.get(number));
				}
			}
			responseData = data;
		}
		//调用解封接口
		if(unfreezeNums.size()>0){
			unfreezeNumsJson.put("yrpg_mainids",unfreezeNums);
			unfreezeDataJson.put("data",unfreezeNumsJson);
			//调用接口
			OpenApiResult adminunfreeze = OpenApiSdkUtil.invoke("/v2/yrpg/base/bos_adminorg/adminunfreeze", unfreezeDataJson, new HashMap<>());

			unfreezeUpdateDataJson.put("data",unfreezeData);
			//调用接口
			OpenApiResult adminsave = OpenApiSdkUtil.invoke("/v2/yrpg/base/bos_adminorg/adminsave", unfreezeUpdateDataJson, new HashMap<>());
			ApiSaveResult data = (ApiSaveResult)adminsave.getData();
			//根据查询更新操作返回值结构，将保存返回值组装
//			setResponseSaveData(data,responseData,isfreeze);
			AssembleResponseUtil.assembleResult(data,responseData,isfreeze);

		}
		//调用保存接口
		if(adminData.size()>0){
			saveDataJson.put("data",adminData);
			//调用接口
			OpenApiResult adminsave = OpenApiSdkUtil.invoke("/v2/yrpg/base/bos_adminorg/adminsave", saveDataJson, new HashMap<>());
			ApiSaveResult data = (ApiSaveResult)adminsave.getData();
			//根据保存更新操作返回值结构，将保存返回值组装
//			setResponseSaveData(data,responseData,isfreeze);
			AssembleResponseUtil.assembleResult(data,responseData,isfreeze);

		}
		if(newfreezeData.size()>0){
			AssembleResponseUtil.addfreezeResult(newfreezeData,responseData);
		}
		//多类型接口 过滤默认为空
		responseData.setFilter("");
		finalResult.setData(responseData);
		return finalResult;
	}



}